﻿using Triangulacya;
using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace WindowsFormsApplication6
{
    public partial class Form1 : Form
    {
        private Pen polygonPen; //для рисования контура многоугольника
        private SolidBrush trianBrush; //для рисования треугольников

        private Graphics g;
        private Bitmap bitmap;

        private Polygon polygon; //многоугольник

        private float translateX, translateY, scale;


        public Form1()
        {
            InitializeComponent();
        }

        private void drawPolygon() //рисует многоугольник
        {
            g.TranslateTransform(translateX, translateY); //центрируем
            g.ScaleTransform(scale, scale); //масштабируем

            //рисуем контур
            PointF[] points = polygon.getPoints();
            g.DrawLines(polygonPen, points);
            g.DrawLine(polygonPen, points[0], points[points.Length - 1]);

            GraphicsPath p = new GraphicsPath();
            Triangle[] trians = polygon.getTriangles();

            if (trians == null)
                return;

            Random rand = new Random();

            //рисуем треугольники
            for (int i = 0; i < trians.Length; i++)
            {
                Triangle t = trians[i];

                p.Reset();
                p.AddLine(t.getA(), t.getB());
                p.AddLine(t.getB(), t.getC());
                p.AddLine(t.getC(), t.getA());

                trianBrush.Color = Color.FromArgb(rand.Next(25, 225), rand.Next(25, 225), rand.Next(25, 225)); //случайный цвет
                g.FillPath(trianBrush, p);
            }
        }

        private void draw()
        {
            bitmap = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            g = Graphics.FromImage(bitmap);
            g.SmoothingMode = SmoothingMode.AntiAlias;

            drawPolygon();

            pictureBox1.Image = bitmap;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            polygonPen = new Pen(Color.Black, 1);
            trianBrush = new SolidBrush(Color.Red);

            float[] vertex = new float[]
            {
                370.48544f, 279.65497f,
                160.86003f, 106.81781f,
                262.63522f, 390.5683f,
                61.91401f, 433.53473f,
                358.50208f, 530.45905f,
                160.7767f, 869.19434f,
                412.4272f, 624.3856f,
                439.38974f, 942.13745f,
                528.2663f, 601.40356f,
                672.0666f, 797.2506f,
                598.16925f, 425.54095f,
                673.22003f, 276.6573f,
                427.4064f, 420.54486f,
                660.08325f, 124.77597f,
                358.50208f, 356.59485f,
                507.29544f, 95.798584f
            }; //х и y координаты многоугольник против часовой стрелки

            //вершины многоугольника против часовой стрелки
            polygon = new Polygon(vertex);

        //разместим многоугольник на весь экран
            //масштабирование
            PointF[] points = polygon.getPoints();

            float minX = int.MaxValue, minY = int.MaxValue;
            float maxX = int.MinValue, maxY = int.MinValue;
            foreach (PointF p in points)
            {
                minX = Math.Min(minX, p.X);
                minY = Math.Min(minY, p.Y);
                maxX = Math.Max(maxX, p.X);
                maxY = Math.Max(maxY, p.Y);
            }

            float width = maxX - minX;
            float height = maxY - minY;

            float scaleX = pictureBox1.Width / width;
            float scaleY = pictureBox1.Height / height;
            scale = Math.Min(scaleX, scaleY) * 0.95f; //коэффициент масштабирования

            //центрирование многоугольника
            translateX = (pictureBox1.Width - width * scale) / 2 - minX * scale;
            translateY = (pictureBox1.Height - height * scale) / 2 - minY * scale;

            draw();

            //площадь многоугольника
            double area = polygon.getArea();
            if (area == 0)
                label.Text = "Многоугольник задан неверно!";
            else
                label.Text = "Площадь = " + area.ToString("0.000");
            //(area * scale).ToString("0.000");

        }
    }
}
